@@ -1,4 +1,5 @@
 from __future__ import absolute_import
 # Copyright (c) 2010-2016 openpyxl
 
-from .cell import *
+from .cell import Cell, WriteOnlyCell
+from .read_only import ReadOnlyCell
