@@ -4,7 +4,10 @@ from __future__ import absolute_import
 """Workbook is the top-level container for all document information."""
 
 from openpyxl.compat import deprecated
+from openpyxl.compat import OrderedDict
 from openpyxl.worksheet import Worksheet
+from openpyxl.worksheet.read_only import ReadOnlyWorksheet
+from openpyxl.worksheet.copier import WorksheetCopy
 
 from openpyxl.utils.indexed_list import IndexedList
 from openpyxl.utils.datetime  import CALENDAR_WINDOWS_1900
@@ -13,32 +16,43 @@ from openpyxl.utils.exceptions import ReadOnlyWorkbookException
 from openpyxl.writer.write_only import WriteOnlyWorksheet, save_dump
 from openpyxl.writer.excel import save_workbook
 
-from openpyxl.styles.styleable import StyleArray
+from openpyxl.styles.cell_style import StyleArray
 from openpyxl.styles.named_styles import NamedStyle
+from openpyxl.styles.differential import DifferentialStyleList
 
 from openpyxl.chartsheet import Chartsheet
-from . names.named_range import NamedRange
-from . properties import DocumentProperties, DocumentSecurity
+from .defined_name import DefinedName, DefinedNameList
+from openpyxl.packaging.core import DocumentProperties
+from openpyxl.packaging.relationship import RelationshipList
+from .protection import DocumentSecurity
+
+from openpyxl.xml.constants import (
+    XLSM,
+    XLSX,
+    XLTM,
+    XLTX
+)
 
 
 class Workbook(object):
     """Workbook is the container for all other parts of the document."""
 
+    _read_only = False
+    _data_only = False
+    _keep_links = True
+    template = False
+    path = "/xl/workbook.xml"
+
     def __init__(self,
-                 optimized_write=False,
-                 encoding='utf-8',
-                 guess_types=False,
-                 data_only=False,
-                 read_only=False,
-                 write_only=False):
+                 write_only=False,
+                 ):
         self._sheets = []
         self._active_sheet_index = 0
-        self._named_ranges = []
+        self.defined_names = DefinedNameList()
         self._external_links = []
         self.properties = DocumentProperties()
         self.security = DocumentSecurity()
-        self.__write_only = write_only or optimized_write
-        self.__read_only = read_only
+        self.__write_only = write_only
         self.shared_strings = IndexedList()
 
         self._setup_styles()
@@ -46,19 +60,16 @@ class Workbook(object):
         self.loaded_theme = None
         self.vba_archive = None
         self.is_template = False
-        self._differential_styles = []
-        self._guess_types = guess_types
-        self.data_only = data_only
-        self._drawings = []
-        self._charts = []
-        self._images = []
+        self._differential_styles = DifferentialStyleList()
         self.code_name = None
         self.excel_base_date = CALENDAR_WINDOWS_1900
-        self.encoding = encoding
+        self.encoding = "utf-8"
 
         if not self.write_only:
             self._sheets.append(Worksheet(self))
 
+        self.rels = RelationshipList()
+
 
     def _setup_styles(self):
         """Bootstrap styles"""
@@ -68,6 +79,7 @@ class Workbook(object):
         from openpyxl.styles.fonts import DEFAULT_FONT
         from openpyxl.styles.protection import Protection
         from openpyxl.styles.colors import COLOR_INDEX
+        from openpyxl.styles.named_styles import NamedStyleList
 
         self._fonts = IndexedList()
         self._fonts.add(DEFAULT_FONT)
@@ -87,17 +99,26 @@ class Workbook(object):
 
         self._colors = COLOR_INDEX
         self._cell_styles = IndexedList([StyleArray()])
-        self._named_styles = {'Normal': NamedStyle(font=DEFAULT_FONT)}
+        self._named_styles = NamedStyleList()
+        self.add_named_style(NamedStyle(font=DEFAULT_FONT, builtinId=0))
 
 
     @property
     def read_only(self):
-        return self.__read_only
+        return self._read_only
+
+    @property
+    def data_only(self):
+        return self._data_only
 
     @property
     def write_only(self):
         return self.__write_only
 
+    @property
+    def keep_links(self):
+        return self._keep_links
+
     @deprecated("Use the .active property")
     def get_active_sheet(self):
         """Returns the current active sheet."""
@@ -126,7 +147,7 @@ class Workbook(object):
             raise ReadOnlyWorkbookException('Cannot create new sheet in a read-only workbook')
 
         if self.write_only :
-            new_ws = WriteOnlyWorksheet(parent_workbook=self, title=title)
+            new_ws = WriteOnlyWorksheet(parent=self, title=title)
         else:
             new_ws = Worksheet(parent=self, title=title)
 
@@ -137,7 +158,7 @@ class Workbook(object):
     def _add_sheet(self, sheet, index=None):
         """Add an worksheet (at an optional index)."""
 
-        if not isinstance(sheet, (Worksheet, Chartsheet)):
+        if not isinstance(sheet, (Worksheet, WriteOnlyWorksheet, Chartsheet)):
             raise TypeError("Cannot be added to a workbook")
 
         if sheet.parent != self:
@@ -149,20 +170,27 @@ class Workbook(object):
             self._sheets.insert(index, sheet)
 
 
-    def remove_sheet(self, worksheet):
+    def remove(self, worksheet):
         """Remove a worksheet from this workbook."""
         self._sheets.remove(worksheet)
 
 
+    @deprecated("Use wb.remove(worksheet) or del wb[sheetname]")
+    def remove_sheet(self, worksheet):
+        """Remove a worksheet from this workbook."""
+        self.remove(worksheet)
+
+
     def create_chartsheet(self, title=None, index=None):
         if self.read_only:
             raise ReadOnlyWorkbookException("Cannot create new sheet in a read-only workbook")
         cs = Chartsheet(parent=self, title=title)
 
-        self._add_sheet(cs, index=None)
+        self._add_sheet(cs, index)
         return cs
 
 
+    @deprecated("Use wb[sheetname]")
     def get_sheet_by_name(self, name):
         """Returns a worksheet by its name.
 
@@ -175,9 +203,16 @@ class Workbook(object):
     def __contains__(self, key):
         return key in set(self.sheetnames)
 
+
+    def index(self, worksheet):
+        """Return the index of a worksheet."""
+        return self.worksheets.index(worksheet)
+
+
+    @deprecated("Use wb.index(worksheet)")
     def get_index(self, worksheet):
         """Return the index of the worksheet."""
-        return self.worksheets.index(worksheet)
+        return self.index(worksheet)
 
     def __getitem__(self, key):
         """Returns a worksheet by its name.
@@ -193,17 +228,19 @@ class Workbook(object):
 
     def __delitem__(self, key):
         sheet = self[key]
-        self.remove_sheet(sheet)
+        self.remove(sheet)
 
     def __iter__(self):
         return iter(self.worksheets)
 
+
+    @deprecated("Use wb.sheetnames")
     def get_sheet_names(self):
         return self.sheetnames
 
     @property
     def worksheets(self):
-        return [s for s in self._sheets if isinstance(s, Worksheet)]
+        return [s for s in self._sheets if isinstance(s, (Worksheet, ReadOnlyWorksheet, WriteOnlyWorksheet))]
 
     @property
     def chartsheets(self):
@@ -220,31 +257,70 @@ class Workbook(object):
         """
         return [s.title for s in self._sheets]
 
-    def create_named_range(self, name, worksheet, range, scope=None):
+    def create_named_range(self, name, worksheet=None, value=None, scope=None):
         """Create a new named_range on a worksheet"""
-        named_range = NamedRange(name, [(worksheet, range)], scope)
-        self.add_named_range(named_range)
+        defn = DefinedName(name=name, localSheetId=scope)
+        if worksheet is not None:
+            defn.value = "{0}!{1}".format(worksheet.title, value)
+        else:
+            defn.value = value
 
+        self.defined_names.append(defn)
+
+
+    def add_named_style(self, style):
+        """
+        Add a named style
+        """
+        style.xfId = self._named_styles.add(style)
+        style.bind(self)
+
+
+    @property
+    def named_styles(self):
+        """
+        List available named styles
+        """
+        return self._named_styles.names
+
+
+    @deprecated("Use workbook.defined_names.definedName")
     def get_named_ranges(self):
         """Return all named ranges"""
-        return self._named_ranges
+        return self.defined_names.definedName
 
+
+    @deprecated("Use workbook.defined_names.append")
     def add_named_range(self, named_range):
         """Add an existing named_range to the list of named_ranges."""
-        self._named_ranges.append(named_range)
+        self.defined_names.append(named_range)
 
+
+    @deprecated("Use workbook.defined_names[name]")
     def get_named_range(self, name):
         """Return the range specified by name."""
-        requested_range = None
-        for named_range in self._named_ranges:
-            if named_range.name == name:
-                requested_range = named_range
-                break
-        return requested_range
+        return self.defined_names[name]
 
+
+    @deprecated("Use del workbook.defined_names[name]")
     def remove_named_range(self, named_range):
         """Remove a named_range from this workbook."""
-        self._named_ranges.remove(named_range)
+        del self.defined_names[named_range]
+
+
+    @property
+    def mime_type(self):
+        """
+        The mime type is determined by whether a workbook is a template or
+        not and whether it contains macros or not. Excel requires the file
+        extension to match but openpyxl does not enforce this.
+
+        """
+        ct = self.template and XLTX or XLSX
+        if self.vba_archive:
+            ct = self.template and XLTM or XLSM
+        return ct
+
 
     def save(self, filename):
         """Save the current workbook under the given `filename`.
@@ -261,3 +337,29 @@ class Workbook(object):
             save_dump(self, filename)
         else:
             save_workbook(self, filename)
+
+
+    @property
+    def style_names(self):
+        """
+        List of named styles
+        """
+        return [s.name for s in self._named_styles]
+
+
+    def copy_worksheet(self, from_worksheet):
+        """Copy an existing worksheet in the current workbook
+        :warning: This function cannot copy worksheets between workbooks.
+        worksheets can only be copied within the workbook that they belong
+
+        :param from_worksheet: the worksheet to be copied from
+        :return: copy of the initial worksheet
+        """
+        if self.__write_only or self._read_only:
+            raise ValueError("Cannot copy worksheets in read-only or write-only mode")
+
+        new_title = "{0} Copy".format(from_worksheet.title)
+        to_worksheet = self.create_sheet(title=new_title)
+        cp = WorksheetCopy(source_worksheet=from_worksheet, target_worksheet=to_worksheet)
+        cp.copy_worksheet()
+        return to_worksheet
